% The root directory in input may contain several folders, each one
% holding one mat file.
% PLEASE DO NOTE: The name of the containing folder MUST be the same of
% the mat file!!
% Each mat file MUST contain at least two structures: the data matrix [M x N]
% containing M observations and the corresponding N features, and
% the array containing the M class labels. The data matrix MUST be named
% data_"name_of_the_dataset". The column label array should be named "lbs".
% Input parameters:
% test_dir_path=dataset root folder full path
% k=the k-fold parameter
% the ranges of the epsilon, delta and lambda parameters and the
% step size.

%% Principal function. Start the test over all the datasets

function RunTestSet(test_dir_path,result_path,maxiter,train_perc,delta_vector,lambda_vector,epsilon_vector)
    % Moves to the test path
    macro_dir=pwd;
    cd (test_dir_path);
    % Cycles on dataset folders, skipping hidden (.xxx) files
    folders = dir; % equal
    folders = folders([folders.isdir]); % equal
    folders(strncmp({folders.name}, '.', 1)) = [];

    [numdir, ~]=size(folders);
    if (numdir==0)
        error('Folder is empty!! Program terminated');
    end
    % the results array
    res_struct = struct([]);

    for idir=1:numdir
        test_name=folders(idir).name;
        cd (test_name);
        msg = sprintf('Started testing on the %d dataset. Name: %s', idir, test_name);
        disp(msg);
        come_back=pwd;
        cd(macro_dir);
        results =  runRTRTest(macro_dir,test_dir_path,test_name,maxiter,delta_vector,lambda_vector,epsilon_vector);
        disp('... test set finished. Saving results and cleaning up.');
        %save the results
        cd(result_path);
        
        name = sprintf('%s_results',test_name);
        
        save(name,'results');
        
        cd(come_back);
        
        clear name lbs data_matrix S;
        cd ..;
    end
    cd(macro_dir);
end

