function [class,acc,Z,W,obj]=regec2_linear(train,train_l,test,test_l,delta,classic)
% Authors:
% Mario Guarracino, Claudio Cifarelli, Onur Seref, Panos Pardalos
% version 2.0
% 2005
%
%  [class,acc,Z,W,T1] = regec2_linear(train,train_l,test,test_l,delta)
% Regec2 classifies two sets of points after a training phase. It implements
% a classification algorithm based on generalized eigenvalue problems.
%
% Input variables
% train:    (input)  contains all training points;
%
% train_1:  (input)  vector of classification labels.
%
% test:     (input)  contains all testing points;
%
% test_1:   (input)  vector of classification labels.
%
% sigma:    (input)  gaussian kernel parameter (denominator). 
% 
% delta:    (input)  regularization parameter for the hyperplanes
%                    (usually 1e-03 - 1e-04 - 1e-05 )
% 
% Output variables
% class:    (output) vector of all evaluated classification labels.
%
% acc:      (output) accuracy of computed classification.
%
% Z:        (output) contains distance from first and last eigenvector and  
%                    classification labels for all test points. 
% W:        (output) contains the coefficients of the two planes
%
% load training data in A and B

% ind1=(train_l==min(train_l));
% ind2=(train_l==max(train_l));
% train_l(ind1,1)=-1;
% train_l(ind2,1)=1;
% 
% ind1=(test_l==min(test_l));
% ind2=(test_l==max(test_l));
% test_l(ind1,1)=-1;
% test_l(ind2,1)=1;
% 
% A=train(train_l==1,:);
% B=train(train_l==-1,:);

global CLASS;

    A=train(train_l==CLASS(1),:);
    B=train(train_l==CLASS(2),:);
    
C=[A;B];

% Build left and right matrices to be used in generalize eigenvalue
% problems solution using a gaussian kernel
g=[A, -ones(size(A,1),1)];
h=[B, -ones(size(B,1),1)];
G1=g'*g;
H1=h'*h;
T=G1;%diag(diag(G1),0);
U=H1;%diag(diag(H1),0);

% Build planes
G = G1 + delta*U;
H = H1 + delta*T;
G=(triu(G,1)' + triu(G,0));
H=(triu(H,1)' + triu(H,0));
[V1,D1] = eig(G,H);
[m1,imin1] = min( diag(abs(real(D1))) );
% Check for fartherest hyperplane  from other class
imin=find(abs(diag(abs(real(D1)))-m1)<eps);
% mol_min=numel(imin);
% best=0;
% imin1=imin(1);
% for i=1:mol_min
%     dist_max=abs(B*V1(1:end-1,imin(i))-V1(end,imin(i)))/norm(V1(1:end-1,imin(i)));
%     if min(dist_max)>best
%         best=dist_max;
%         imin1=imin(i);
%     end
% end
W=V1(:,imin1);

[m2,imax2] = max( diag(abs(real(D1))) );
% imax=find(abs(diag(abs(real(D1)))-m2)<eps);
% mol_max=numel(imax);
% best=0;
% imax2=imax(1);
% for i=1:mol_max
%     dist_max=abs(A*V1(1:end-1,imax(i))-V1(end,imax(i)))/norm(V1(1:end-1,imax(i)));
%     if min(dist_max)>best
%         best=dist_max;
%         imax2=imax(i);
%     end
% end
W=[W, V1(:,imax2)];

obj=[m1,1/m2];

% aa=sort(diag(abs(D1)));
% sizeD1=size(D1,1);
% T1=[ aa(1) aa(sizeD1) abs(aa(1) - aa(2))/abs(aa(1)), abs(aa(sizeD1)-aa(sizeD1-1))/abs(aa(sizeD1))];

% Classify test points
n=size(A,2);
Z=[];

    if nargin<6
        if ((size(A,1)<size(A,2))||(size(B,1)<size(B,2)))
            classic=1;
        else
            classic=0;
        end
    end

    if classic == 0     % %     WITH PROJECTION
        tempTRclass=zeros(size(train_l));
        for i=1:2
            tempTRclass(train_l==CLASS(i))=i;
        end
        tempTESTclass=PredictMRG(train,tempTRclass, W, test);
        class=zeros(size(test_l));
        for i=1:2
            class(tempTESTclass==i) = CLASS(i);
        end
    else                % %     CLASSICAL
        z1=(abs(test*W(1:n,1)-W(n+1,1)).^2)/(norm(W(1:n,1))^2);
        z2=(abs(test*W(1:n,2)-W(n+1,2)).^2)/(norm(W(1:n,2))^2);
        class=zeros(size(test_l));
        for i=1:size(test,1)
            if(z1(i)<z2(i))
                class(i)=CLASS(1);
            else
                class(i)=CLASS(2);
            end
        end
    end
        

    acc=sum((class-test_l)==0)/size(class,1);
        
W=[V1(:,imin1)/norm(V1(:,imin1)), V1(:,imax2)/norm(V1(:,imax2))];

global tol;
reltol=1/sqrt(n+1)*tol;
W(1:end-1,:) = W(1:end-1,:).*(abs(W(1:end-1,:))>=reltol);