function [x,fx,RQvar,L1norm,dist] = rtr_l1approx(b,w,delta,params)

% EDIT: Marco Viola
% 
% About: RTR - Riemannian Trust-Region
% (C) 2004-2007, P.-A. Absil, C. G. Baker, K. A. Gallivan
% Florida State University
% School of Computational Science

   % set pointers for functions required by RTR
   n = size(b,2);
   fns.R     = @(x,eta)R(x,eta);     % retraction
   fns.g     = @(x,eta,zeta)g(eta,zeta); % Riemannian metric
   fns.proj  = @proj;   % projection onto tangent plane from nearby
   fns.randT = @randTg;  % random vector on the tangent plane;
   fns.f     = @(x,var)f(b,w,delta,x);     % objective function
   fns.fgrad = @(x,var)grad(b,w,delta,x,var);  % gradient of f
   fns.fhess = @(x,eta,var)H(b,w,delta,x,eta,var);     % Hessian of f

   % set parameters for RTR algorithm
%    params.useRand   = 1;
%    params.debug     = 1;
   params.verbosity = 0;
   params.max_inner = n*(n-1)/2;
   if ~isfield(params,'Delta_bar')
      params.Delta_bar = n/10;     
   end
   if ~isfield(params,'Delta0'),
      params.Delta0    = params.Delta_bar/5;
   end
   if ~isfield(params,'x0')
       params.x0    = zeros(n,1);
       params.x0(n) = 1; 
       params.x0 = rand(n,1);
   

       for i=1:20
        temp=w*params.x0;
        params.x0=w'*temp;
        params.x0=params.x0/norm(params.x0);
       end
   end
      
   [x,fx,RQvar,L1norm] = Xrtr(fns,params);   % apply the RTR to solve the problem.
   
   dist = norm(params.x0-x)/norm(params.x0);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function retract = R(x,eta)
    retract=(x+eta)/norm(x+eta);
    

function ez = g(eta,zeta)
    ez=eta'*zeta; 

function z=randTg(x)
    z=rand(size(x));
    z=proj(x,z);

function project = proj(x,z)
   project = z - (z'*x)*x;

function [fval,var,RayQ] = f(b,w,delta,x)

   global lambda;
   global epsilon;
   
   bx=b*x;
   wx=w*x;
   
   phib=bx'*bx;
   phiw=wx'*wx;
   RayQ=(phib+delta*phiw)/(phiw+delta*phib);
   fval=RayQ+lambda*sum(sqrt(x.^2+epsilon));

   var=struct('bx',bx,'wx',wx,'phib',phib,'phiw',phiw);
   
 
function grad = grad(b,w,delta,x,var)

   global lambda;
   global epsilon;
   
   if nargin>4
       bx   = var.bx;
       wx   = var.wx;
       phiW = var.phiw+delta*var.phib;
       phiB = var.phib+delta*var.phiw;
   else
       bx   = b*x;
       wx   = w*x;
       phiW = wx'*wx+delta*bx'*bx;
       phiB = bx'*bx+delta*wx'*wx;
   end
   
   Bx=b'*bx;
   Wx=w'*wx;
   
   grad=1/(phiW)*(Bx+delta*Wx-(phiB/phiW)*(Wx+delta*Bx));
   grad=2*grad;
   n=size(grad,1);
   for i=1:n-1
       grad(i)=grad(i)+lambda*x(i)/sqrt(x(i)^2+epsilon);
   end
   grad=grad-grad'*x*x;
   
function hess = H(b,w,delta,x,eta,var)

   global lambda;
   global epsilon;

   if nargin>5
       bx   = var.bx;
       wx   = var.wx;
       phiW = var.phiw+delta*var.phib;
       phiB = var.phib+delta*var.phiw;
   else
       bx   = b*x;
       wx   = w*x;
       phiW = wx'*wx+delta*bx'*bx;
       phiB = bx'*bx+delta*wx'*wx;
   end
   
   tempBx=b'*bx;
   tempWx=w'*wx;
   
   Bx=tempBx+delta*tempWx;
   Wx=tempWx+delta*tempBx;

   eta=eta-(eta'*x)*x;
   weta=Wx'*eta;
   beta=Bx'*eta;
   
%   Rayleigh quotient Hessian
   hess = 1/phiW*(b'*(b*eta)-phiB/phiW*w'*(w*eta))+4*phiB/(phiW^3)*Wx*weta-2/phiW^2*(Bx*weta+Wx*beta);
   hess = 2*hess;
   hess = hess-(hess'*x)*x;
    
%   eps-L1 Hessian
   if (lambda~=0)
       n=max(size(eta));
       H_eps=lambda*epsilon*eye(n);
       H_eps(n,n)=0;
       
       for i=1:n
           H_eps(i,i)=H_eps(i,i)/sqrt(x(i)^2+epsilon)^3;
       end
       
       temp=H_eps*eta;
       
       temp=temp-(temp'*x)*x-lambda*x'*([x(1:n-1);0]./sqrt(x.^2+epsilon))*eta;

       hess=hess+temp;
   end