% This function cycles on the outer parameters epsilon, delta, lambda.
% It starts a k-fold cross validation for  each triple of values

function results =  runRTRTest(macro_dir,test_dir_path,test_name,maxiter,delta_vector,lambda_vector,epsilon_vector)

global numclasses;
global CLASS;

cd(test_dir_path);
cd(test_name);

[nome_train,err]=   sprintf('%strain%d.asc',test_name,1);
[nome_train_l,err]= sprintf('%strain_l%d.asc',test_name,1);


train=load(nome_train);
train_l=load(nome_train_l);
cd(macro_dir);
CLASS = sort(unique(train_l));
numclasses = max(size(CLASS));


dimlam=max(size(lambda_vector));
dimeps=max(size(epsilon_vector));
dimdel=max(size(delta_vector));
dim_lam_eps=dimeps*dimdel;

% Result matrices initialization
Accuracy=zeros(dimdel,dimlam,dimeps);
Acc_StdDev=zeros(dimdel,dimlam,dimeps);

Accuracy_REG=zeros(dimdel,1);
Acc_StdDev_REG=zeros(dimdel,1);

FeatureSelectedClass0       = zeros(size(train,2),dimdel,dimlam,dimeps);
FeatureSelectedClass1       = zeros(size(train,2),dimdel,dimlam,dimeps);
FeatureSelectedClass0_ReGEC = zeros(size(train,2),dimdel);
FeatureSelectedClass1_ReGEC = zeros(size(train,2),dimdel);

Sparsity=zeros(dimdel,dimlam,dimeps);
Sparsity_REG=zeros(dimdel,1);

% Now cicles over delta(del_ind), lambda(lam_ind) and epsilon(eps_ind)

    ZEROS_REG=zeros(dimdel,maxiter);

for del_ind=1:dimdel
    
    delta  = delta_vector(del_ind);
    
    ACC_RTR=zeros(dim_lam_eps,maxiter);
    ACC_REG=zeros(del_ind,maxiter);
    ZEROS_RTR=zeros(dim_lam_eps,maxiter);
    

    for i=1:maxiter
        
        cd(test_dir_path);
        cd(test_name);
        
        [nome_train,err]=   sprintf('%strain%d.asc',test_name,i);
        [nome_train_l,err]= sprintf('%strain_l%d.asc',test_name,i);
        [nome_test,err]=    sprintf('%stest%d.asc',test_name,i);
        [nome_test_l,err]=  sprintf('%stest_l%d.asc',test_name,i);


        train=load(nome_train);
        train_l=load(nome_train_l);
        test=load(nome_test);
        test_l=load(nome_test_l);
        cd(macro_dir);
        
        [stdtrain,mu,sigma] = zscore(train,1);
        
        stdtrain = stdtrain(:,sigma>0);
        
        stdtest=zeros(size(test));
        for ind=1:size(test,2)
            stdtest(:,ind)=(test(:,ind)-mu(ind))/sigma(ind);
        end
        
        feat_ind = sigma>0;
        stdtest = stdtest(:,feat_ind);
        
        start  = struct('W',0,'obj',0);
        
        % % % %         linear_ReGEC
        msg = sprintf('\nStarted [delta = %d] calculation with linear_ReGEC on package %d',delta,i);
        disp(msg);

        if numclasses==2
            [class,acc,Z,W,REG_obj]=regec2_linear(stdtrain,train_l,stdtest,test_l,delta);
            
            for tmp=1:size(train,2)
                if feat_ind(tmp)==1
                    FeatureSelectedClass0_ReGEC(tmp,del_ind) = FeatureSelectedClass0_ReGEC(tmp,del_ind) + (abs(W(tmp,1))>0);
                    FeatureSelectedClass1_ReGEC(tmp,del_ind) = FeatureSelectedClass1_ReGEC(tmp,del_ind) + (abs(W(tmp,2))>0);
                end
            end
        else
            error('Number of classes greater than two!! Program terminated');
        end
        
        ACC_REG(del_ind,i) = acc;
        
        msg = sprintf('\n Regec accuracy at iteration %d is %d', i,acc);
        disp(msg);
        
        for cla=1:numclasses
            W_r(:,cla)=W(:,cla)/norm(W(:,cla));
        end

        ZEROS_REG(del_ind,i) = sum(sum(abs(W(1:end-1,:))==0,2)==size(W,2))/(size(W,1)-1);

        L0=mean(REG_obj);

        
        clear start;
        start = struct('W',W_r,'obj',REG_obj);
        
        if ~(size(lambda_vector,2)==1 && lambda_vector(1)==0)
            for lam_ind=1:dimlam
                lambda=lambda_vector(lam_ind);
                for eps_ind=1:dimeps
                    epsilon=epsilon_vector(eps_ind);
                    msg = sprintf('\nStarted [ delta = %d, lambda= %d , epsilon= %d] calculation with RTR on package %d', delta, lambda, epsilon,i);
                    disp(msg);
                   
                    lambda=lambda*L0;

                    if numclasses==2
                        [classes,W,~,sparsity,RQvariation,L1norm,dist] = RTR_2class       (stdtrain,train_l,stdtest,test_l,delta,lambda,epsilon,start);
                        for tmp=1:size(train,2)
                            if feat_ind(tmp)==1
                                FeatureSelectedClass0(tmp,del_ind,lam_ind,eps_ind) = FeatureSelectedClass0(tmp,del_ind,lam_ind,eps_ind) + (abs(W(tmp,1))>0);
                                FeatureSelectedClass1(tmp,del_ind,lam_ind,eps_ind) = FeatureSelectedClass1(tmp,del_ind,lam_ind,eps_ind) + (abs(W(tmp,2))>0);
                            end
                        end
                    else
                        error('Number of classes greater than two!! Program terminated');
                    end
                    
                    ZEROS_RTR(lam_ind*dimeps+eps_ind,i) = sparsity;
                    acc = sum((classes-test_l)==0)/size(classes,1);
                    ACC_RTR(lam_ind*dimeps+eps_ind,i) = acc;

                    msg = sprintf('\n Regec L1 accuracy at iteration %d is %d', i,acc);
                    disp(msg);
                end
            end
        end
    end
    
    Accuracy_REG(del_ind)   = mean(ACC_REG(del_ind,:));
    Acc_StdDev_REG(del_ind) = std(ACC_REG(del_ind,:));
    
    Sparsity_REG(del_ind)   = mean(ZEROS_REG(del_ind,:));
    
    if ~(size(lambda_vector,2)==1 && lambda_vector(1)==0)
        for lam_ind=1:dimlam
            for eps_ind=1:dimeps
                Accuracy(del_ind,lam_ind,eps_ind)   = mean(ACC_RTR(lam_ind*dimeps+eps_ind,:));
                Acc_StdDev(del_ind,lam_ind,eps_ind) = std(ACC_RTR(lam_ind*dimeps+eps_ind,:));
                Sparsity(del_ind,lam_ind,eps_ind)   = mean(ZEROS_RTR(lam_ind*dimeps+eps_ind,:));
            end
        end
    end
end
% % the results

results = struct('Accuracy',Accuracy,'Acc_StdDev',Acc_StdDev,'Accuracy_ReGEC',Accuracy_REG,'Acc_StdDev_ReGEC',Acc_StdDev_REG,...
    'Sparsity',Sparsity,'FeatureSelectedClass0',FeatureSelectedClass0/maxiter,'FeatureSelectedClass1',FeatureSelectedClass1/maxiter,...
    'FeatureSelectedClass0_ReGEC',FeatureSelectedClass0_ReGEC/maxiter,'FeatureSelectedClass1_ReGEC',FeatureSelectedClass1_ReGEC/maxiter,...
    'Sparsity_ReGEC',Sparsity_REG);
end